%------------------------------------------------------------
%               calculatespeed.m
%               Hege Eskild
%               Spring 2014
%
% This is the underlying function in optfuel2. It calculates the 
% attainable speed for a given sea state. The added resistance
% from waves is defined as a percenteage incease of the calm water
% resitance. The added resistance from wind is calculated. 
% Propulsion factors are find by using cubic spline interpolation
% from the open water test and the propulsion test
%
%------------------------------------------------------------
%   Variable name               Definition
%----------------------Input-----------------------------
%   T1                          Mean wave period
%   Hs                          Significant wave height
%   V                           Vessel speed
%   head                        relative heading between waves and 
%                               vessel
%   Pb                          Desired brake power
%   D                           Propeller diameter
%   Eta_M                       Mechanical efficiency
%   Eta_R                       Relative rotational efficiency
%   AddedRes                    Matrix with increased resistance 
%                               due to waves
%   C_wind                      Matrix with wind drag coefficient 
%                               for different headings
%   R_calm                      Calm water resistance matrix
%   OpenWater                   Matrix with open water values
%   U_wind                      U component of wind 
%   V_wind                      V component of wind 
%   C_wake                      Wake coefficient
%   C_t                         Thrust deduction coefficient
%---------------------Output--------------------------------
%   V                           Vessel velocity
%   head_wind                   Heading of wind
%   Vel_wind                    Wind velocity



function [V,head_wind,Vel_wind]=calculateSpeed(Hs,T1,head,U_wind,...
         V_wind,AddedRes,C_wind,R_calm,OpenWater,C_wake,C_t,A_T,...
         Eta_M,Eta_R,D,Pb,az)

Tp=1.3*T1;
V=19;
P_B=0;

rho_air=1.2250;

%finding resistance increase due to waves

for k=1:length(AddedRes)
    if AddedRes(k,1)>(head-15) && AddedRes(k,1)<(head+15)...
       && AddedRes(k,2)>(Hs-0.5) && AddedRes(k,2)<(Hs+0.5)...
       && AddedRes(k,3)>(Tp-0.5) && AddedRes(k,3)<(Tp+0.5)
          R_w=AddedRes(k,4);
    end
end

%Calculating the wind velocity and heading in a 360 degrees range
Vel_wind=sqrt(U_wind^2+V_wind^2);
head_wind1=atan(abs(U_wind/V_wind))*180/pi();

if V_wind<0 && U_wind>0
    head_wind1=atan(abs(V_wind/U_wind))*180/pi();
    head_wind=head_wind1+90;
elseif V_wind<0 && U_wind<0
    head_wind1=atan(abs(U_wind/V_wind))*180/pi();
    head_wind=head_wind1+180;
elseif V_wind>0 && U_wind<0
    head_wind1=atan(abs(V_wind/U_wind))*180/pi();
    head_wind=head_wind1+270;
else
    head_wind1=atan(abs(U_wind/V_wind))*180/pi();
    head_wind=head_wind1;
end

head_wind;

%Calculating wind heading relative to ship heading
head_wind=head_wind-az-180;
    if head_wind<-180
        head_wind=abs(head_wind+360);
    elseif head_wind<0
        head_wind=abs(head_wind);
    end    
head_wind;
%finding wind resistance coefficient with interpolation
C_Rwind= spline(C_wind(:,1),C_wind(:,2),head_wind);


while P_B<Pb && V<25.5
    V=V+0.02;
    Vs=V*1852/3600;
    %Resistance due to wind
    R_wind=C_Rwind*0.5*rho_air*(Vel_wind^2+2*Vel_wind*Vs)*A_T/1000;

    %Calm water resistance
    %Interpolating restistance to find accurate resistance for 
    %given speed in Kn
    R_C= spline(R_calm(:,1),R_calm(:,2),V);

    R_wave=R_w/100*R_C;

    %Finding total resistance
    R_T=R_wind+R_wave+R_C;

    %Calculating efffective power
    P_E=R_T*Vs;

    %Finding the RPM
    KtoverJ2=R_T*10^3/(1025*(1-C_t)*D^2*Vs^2*(1-C_wake)^2);
    J_full=spline(OpenWater(:,5),OpenWater(:,1),KtoverJ2);
    RPM=60*(1-C_wake)/D*Vs/J_full;
    n=RPM/60;

    %Calculating J
    V_A=Vs*(1-C_wake);
    J=V_A/(n*D);

    Eta_0= spline(OpenWater(:,1),OpenWater(:,4),J);

    %Hull efficiency
    Eta_H=(1-C_t)/(1-C_wake);

    %Calculating the propulsive efficiency
    Eta_D=Eta_H*Eta_0*Eta_R;

    P_D=P_E/Eta_D;

    %Brake power
    P_B=P_D/Eta_M;

end
end