%------------------------------------------------------------
%               gcfuel.m
%               Hege Eskild
%               Spring 2014
%
% In this function the fuel consumption for the great circle 
% route (GCR) is calculated. The GRC in this program is
% is defined as a practical GCR, meaning that the route 
% is constructed of three straight legs and not a circle.
% The output of this function is the fuel consumption and 
% two vectors with lat and lon later used to construct the
% feasible region for the optimal route.
%
%------------------------------------------------------------
%   Variable name               Definition
%----------------------Input-----------------------------
%   GClat                       Four latitude values for the 
%                               practical GCR
%   GClon                       Four longitude values for the 
%                               practical GCR
%   Pb                          Desired brake power
%   bsfc                        Specific fuel consumption (g/kWh)
%   AVspeed                     Matrix with attainable speed for 
%                               different sea states
%                               for different headings
%   lon                         Vector with longitude values from 
%                               GRIB
%   lat                         Vector with latitude values from 
%                               GRIB
%   SWH                         Matrix with Hs from GRIB
%   MWD                         Matrix with mean wave direction 
%                               from GRIB
%   MWP                         Matrix with mean wave period 
%                               from GRIB
%---------------------Output--------------------------------
%   GCFC                        Fuel consumption for GCR
%   GClats                      Latitude vector for GCR
%   GClons                      Longitude vector for GCR
%---------------------Internal--------------------------------
%   i                           Counting integer 
%   k                           Counting integer 
%   Hs                          Significant wave height for 
%                               the given arc
%   T1                          Mean wave period for the  
%                               given arc
%   Tp                          Peak wave period for the  
%                               given arc
%   head                        Relative heading between wave
%                               and vessel or given arc
%   arcTimeS                    Vector with traveled time for
%                               all arc
%   arcFuelS                    Vector with fuel consumption for
%                               all arcs
 
function [GCFC,GClats,GClons]=gcfuel(GClat,GClon,Pb,bsfc,...
          AVspeed,lon,lat,SWH,MWD,MWP);
            
%There are three legs in the practical GCR. The for-loop
%assigns lat and lon values to the points in-between. 
for i=1:3
 
    [arclen1(i,1),az1(i,1)] = distance('rh',GClat(i),...
                              GClon(i),GClat(i+1),GClon(i+1));
    [lat1,lon1] = track2('rh',GClat(i),GClon(i),GClat(i+1),...
                  GClon(i+1),referenceSphere('unit sphere'),...
                  'degrees',round(arclen1(1,1)));
    lats1(:,i)=lat1;
    lons1(:,i)=lon1;
end
 
%Creating on vector for lon and one for lat for the entire route
for i=1:3*length(lats1)-2
    if i<length(lats1)
    lats(i,1)=lats1(i,1);
    lons(i,1)=lons1(i,1);
    elseif i<2*length(lats1)
        lats(i,1)=lats1(i-length(lats1)+1,2);
        lons(i,1)=lons1(i-length(lats1)+1,2);
    else
        lats(i,1)=lats1(i-2*length(lats1)+2,3);
        lons(i,1)=lons1(i-2*length(lats1)+2,3);
    end
end
lats;
lons;
 
%Calculating the fuel consumption for each arc 
for i=1:length(lats)-1
    %Defining arc lenghts for going straight
    [arcStraight(i,1),azS(i,1)]=distance...
    ('gc',lats(i,1),lons(i,1),lats(i+1,1),lons(i+1,1));
    %Finding the index to the latitude and longitude weather vector
    [~, idxlat] = min(abs(lat - lats(i+1,1)));
    [~, idxlon] = min(abs(lon - lons(i+1,1)));
    az = azimuth(lats(i,1),lons(i,1),lats(i+1,1),lons(i+1,1));
    %Finding Hs, T1 and direction for the given arc
    idxtime=round(i/2);
    Hs=SWH(idxtime,idxlat,idxlon);
    if Hs<1 || isnan(Hs)    
        Hs=1;
    end
    T1=MWP(idxtime,idxlat,idxlon);
    if T1<1 || isnan(T1)
        T1=5.4;
    end
    Tp=T1*1.3;
    dir=MWD(idxtime,idxlat,idxlon);
    if dir<1 || isnan(dir)
        dir=90;
    end
    %Finding relative heading between wave and vessel
    head=sqrt(az^2+dir^2-2*az*dir);
        if head>180
            head=360-head;
        end    
    headS(i,1)=head;
    TpS(i,1)=Tp;
    HsS(i,1)=Hs;
 
    %Assigning the speed to the arc for the given weather 
    %condition
    for k=1:length(AVspeed)
        if AVspeed(k,1)>(head-15) && AVspeed(k,1)<(head+15)...
           && AVspeed(k,2)>(Hs-0.5) && AVspeed(k,2)<(Hs+0.5)...
           && AVspeed(k,3)>(Tp-0.5) && AVspeed(k,3)<(Tp+0.5)
              V=AVspeed(k,4);
        end
    end
    VS(i,1)=V;
    Vcalm=23.8;
    %Assiging the fuel cost for the given arc
    arcTimeS(i,1)=deg2nm(arcStraight(i,1))/V;
    arcFuelS(i,1)=Pb*arcTimeS(i,1)*bsfc/1000000; 
    arcTimeCalm(i,1)=deg2nm(arcStraight(i,1))/Vcalm;
    arcFuelCalm(i,1)=Pb*arcTimeCalm(i,1)*bsfc/1000000; 
    
end
GCFC=sum(arcFuelS)
tottime=sum(arcTimeS);
CalmWaterFC=sum(arcFuelCalm);
 
GClats=lats;
GClons=lons;         
            
end

