%------------------------------------------------------------
%               main.m
%               Hege Eskild
%               Spring 2014
%
% This is the main script in the Matlab program including
% several underlying functions. The first part of the script
% in user input. User input is divided into route input,
% including start and end points and weather files, and
% Vessel input if the user wishes to change vessel.
% The second part is the functions, divided into read and 
% Calculating functions. The read functions reads the ShipX
% Input and the GRIB files. The second parts calculates the 
% fuel consumption for GCR and optimal route.
%
% All functions will include a short description and an
% Explanation of the variables. All input and output
% is defined and some of the internal variables. Not 
% all internal variables are described in detail because
% many of them are "help" variables and not significantly 
% important in order to understand the program.
%
%------------------------------------------------------------
%   Variable name               Definition
%----------------------Input-----------------------------
%   startLat                    Latitude coordinate departure
%   startLon                    Longitude coordinate departure
%   endLat                      Latitude coordinate arrival
%   endLon                      Longitude coordinate arrival
%   CalculateWindFromWave       Defines how to calculate wind 
%                               resistance
%   GRIBinput                   Weather file. Can be forecast or 
%                               hindcast in GRIB format
%   ProbSlamming                Seakeeping criterion for slamming
%   ProbGreenWater              Seakeeping criterion for deck 
%                               wetness
%   VAccFP                      Seakeeping criterion for vertical 
%                               acceleration
%   RMSRoll                     Seakeeping criterion for roll
%   NumCenPts                   Number of center points for opt 
%                               route
%   DeltaNM                     Distance in NM between center 
%                               points
%   GClat                       Four latitude values for the 
%                               practical GCR
%   GClon                       Four longitude values for the 
%                               practical GCR
%   TRANSinput(..)              Transfer files from ShipX (.re1)
%   SLinput                     Speed loss input file from ShipX 
%                               (seaway_ship)
%   ADDEDRESinput               Speed loss input file from ShipX 
%                               (seaway_ship.out)
%   Cwindinput                  Wind resistance coefficient input 
%                               file
%   CalmResinput                Calm water resistance input file
%   Pb                          Desired brake power
%   bsfc                        Specific fuel consumption (g/kWh)
%   Xp                          X coordinate of FP (negative from 
%                               CG)
%   freeboard                   freeboard at FP
%   A_T                         Transverse projected area
%   D                           Propeller diameter
%   Eta_M                       Mechanical efficiency
%   Eta_R                       Relative rotational efficiency
%   OpenWaterinput              Open water diagram, text file
%   C_wake                      Wake coefficient
%   C_t                         Thrust deduction coefficient
%                
%---------------------Output--------------------------------
%   AVspeed                     Matrix with attainable speed for 
%                               different sea states
%   NOHEAD                      Number of different headings in 
%                               Avspeed
%   NOTpHs                      Number of Tp and Hs in Avspeed
%   AddedRes                    Matrix with increased resistance 
%                               due to waves
%   C_wind                      Matrix with wind drag coefficient 
%                               for different headings
%   R_calm                      Calm water resistance matrix
%   OpenWater                   Matrix with open water values
%   lon                         Vector with longitude values from 
%                               GRIB
%   lat                         Vector with latitude values from 
%                               GRIB
%   time                        Vector with time values from GRIB
%   Uwind                       Matrix with U component of wind 
%                               from GRIB
%   Vwind                       Matrix with V component of wind 
%                               from GRIB
%   SWH                         Matrix with Hs from GRIB
%   MWD                         Matrix with mean wave direction 
%                               from GRIB
%   MWP                         Matrix with mean wave period 
%                               from GRIB
%   TRANS(..)                   Matrices including transfer
%                               functions for roll and heave/
%                               pitch for different frequencies
%                               heading and velocity.
%   NOHEAD                      Number of heading in transfer
%                               file
%   NOFREQ                      Number of frequencies in
%                               transfer files
%   Lpp                         Length per perpendiculars
%   GCFC                        Fuel consumption for GCR
%   GClats                      Latitude vector for GCR
%   GClons                      Longitude vector for GCR
%   FC                          Fuel consumption for opt. route
 
clear all
close all
 
%---------------------------------------------------------------
% The first part of the main file is for user input, both files 
% and variables
 
%Define start and end point of voyage
startLat=48.5;
startLon=-125;
endLat=38;
endLon=141;
 
%Do you wish to calculate wind velocity from significant wave 
%height? Write 1 for calculating wind from waves, 0 otherwise 
CalculateWindFromWave=1;
 
%Input GRIB file
GRIBinput='janNP.grib';
 
%Define operability criteria; Probability of slamming and 
%green water, as well ass vertical Acc at bow and rms of roll
ProbSlamming=0.03;
ProbGreenWater=0.05;
VAccFP=1.37; %[m/s^2]
RMSRoll=6;  %[deg]
 
%Define number of center points, odd number. Defines the range 
%in which the vessel can travel. 11 center point gives 600 
%nautical miles range at %center of the route
NumCenPts=15;
 
[GClat,GClon]=gcwaypts(startLat,startLon,endLat,endLon,3);
 
 
%Define arclength for calculating fuel consumption
DeltaNM=60;         %Deltalengtt in NM
 
%Define transferinput from ShipX name input.re1
%The functions are calculated with different wave amplitude
%Only include files with the same values for heading and period.
TRANSinputHP=fopen('TRANSHP.txt','r');
TRANSinputH2=fopen('TRANSH2.txt','r');
TRANSinputH4=fopen('TRANSH4.txt','r');
TRANSinputH6=fopen('TRANSH6.txt','r');
TRANSinputH8=fopen('TRANSH8.txt','r');
TRANSinputH10=fopen('TRANSH10.txt','r');
 
%Define speed loss input file from ShipX called seaway_ship 
%(found in therun folder). File is modified. Copy only 
%available speed in waves,including number of headings 
%and number of Hs and Tp
SLinput=fopen('SpeedLoss_ShipX.txt','r');
 
%Define files for calculating the speed loss by separate wind 
%velocity. The speed loss file from ShipX is know the 
%seaway_shipX.out. Remove everything until the defining of 
%the first heading. Define also a calm water resistance .txt 
%file and a wind resistance coefficient file.
ADDEDRESinput='SpeedLoss2_ShipX.txt';
Cwindinput='WindResCoefficient.txt';
CalmResinput='CalmWaterRes.txt';
 
 
%Define constant brake power, arclength for calculating fuel 
%consumption and ship motions and specific fuel consumption
Pb=28710.00;        %Brake power
bsfc=166;           %Brake specific fuel consumption in g/kWh
 
%Define some position of FP Xp, freeboard height and 
%transverse projected area
Xp=-109.77;
freeboard=15.27;
A_T=850;
 
%Input propulsion settings
D=7.5;           %Propeller diameter
Eta_M=0.980;     %Mechanical efficiency
Eta_R=1.01;      %Rotational efficiency
C_wake=0.24;
C_t=0.18;
OpenWaterinput='OpenWater.txt';   %Open water diagram
 
setup_nctoolbox   %if needed
 
%------------------------------------------------------------
%The second part of the main file is calculations
 
%Read ShipX Speed loss files
[AVspeed,NOHEAD,NOTpHs]=speedloss(SLinput);
 
%Reading increase in resistance due to waves, the wind 
%resistance coefficient and calm water resistance
[AddedRes,C_wind,R_calm,OpenWater]=...
increaseaddedres(NOHEAD,NOTpHs,...
ADDEDRESinput,Cwindinput,CalmResinput,OpenWaterinput);
 
%Read GRIB file
[lon,lat,time,Uwind,Vwind,SWH,MWD,MWP]=readgrib(GRIBinput);
 
%Read ShipX transfer files
[TRANSR2,TRANSR4,TRANSR6,TRANSR8,TRANSR10,TRANSHP,NOFREQ,...
NOHEAD,Lpp]=readinput(TRANSinputHP,TRANSinputH2,TRANSinputH4...
,TRANSinputH6,TRANSinputH8,TRANSinputH10);
 
%Calculating GCfuel   
 [GCFC,GClats,GClons]=gcfuel(GClat,GClon,Pb,bsfc,...
 AVspeed,lon,lat,SWH,MWD,MWP); 
 
  if CalculateWindFromWave==1
 %Calculating Optimal route
 [FC]=optfuel(startLat,startLon,endLat,endLon,NumCenPts,Pb,...
 DeltaNM,bsfc,AVspeed,lon,lat,time,Uwind,Vwind,...
 SWH,MWD,MWP,ProbSlamming,ProbGreenWater,VAccFP,...
 RMSRoll,TRANSR2,TRANSR4,TRANSR6,TRANSR8,TRANSR10,TRANSHP,...
 NOFREQ,NOHEAD,Lpp,Xp,freeboard,GClats,GClons)
 
  elseif CalculateWindFromWave==0
 [FC]=optfuel2(startLat,startLon,endLat,endLon,NumCenPts,Pb,...
 DeltaNM,bsfc,AVspeed,lon,lat,time,Uwind,Vwind,...
 SWH,MWD,MWP,ProbSlamming,ProbGreenWater,VAccFP,...
 RMSRoll,TRANSR2,TRANSR4,TRANSR6,TRANSR8,TRANSR10,TRANSHP,...
 NOFREQ,NOHEAD,Lpp,Xp,freeboard,AddedRes,C_wind,R_calm,...
 OpenWater,A_T,Eta_M,C_wake,C_t,Eta_R,D,GClats,GClons)
    end

