%------------------------------------------------------------
%               optfuel2.m
%               Hege Eskild
%               Spring 2014
%
% This is the optimal fuel function for actual wind. The method
% is described in chapter 3 in the thesis. The only difference
% between this function and the optfuel function is that the 
% speed is calculated and not read from the ShipX file.
%
%------------------------------------------------------------
%   Variable name               Definition
%----------------------Input-----------------------------
%   startLat                    Latitude coordinate departure
%   startLon                    Longitude coordinate departure
%   endLat                      Latitude coordinate arrival
%   endLon                      Longitude coordinate arrival
%   ProbSlamming                Seakeeping criterion for slamming
%   ProbGreenWater              Seakeeping criterion for deck 
%                               wetness
%   VAccFP                      Seakeeping criterion for vertical 
%                               acceleration
%   RMSRoll                     Seakeeping criterion for roll
%   NumCenPts                   Number of center points for opt 
%                               route
%   DeltaNM                     Distance in NM between center 
%                               points
%                               practical GCR
%   Pb                          Desired brake power
%   bsfc                        Specific fuel consumption (g/kWh)
%   Xp                          X coordinate of FP (negative from 
%                               CG)
%   freeboard                   freeboard at FP
%   A_T                         Transverse projected area
%   D                           Propeller diameter
%   Eta_M                       Mechanical efficiency
%   Eta_R                       Relative rotational efficiency
%   AVspeed                     Matrix with attainable speed for 
%                               different sea states
%   AddedRes                    Matrix with increased resistance 
%                               due to waves
%   C_wind                      Matrix with wind drag coefficient 
%                               for different headings
%   R_calm                      Calm water resistance matrix
%   OpenWater                   Matrix with open water values
%   C_wake                      Wake coefficient
%   C_t                         Thrust deduction coefficient
%   lon                         Vector with longitude values from 
%                               GRIB
%   lat                         Vector with latitude values from 
%                               GRIB
%   time                        Vector with time values from GRIB
%   Uwind                       Matrix with U component of wind 
%                               from GRIB
%   Vwind                       Matrix with V component of wind 
%                               from GRIB
%   SWH                         Matrix with Hs from GRIB
%   MWD                         Matrix with mean wave direction 
%                               from GRIB
%   MWP                         Matrix with mean wave period 
%                               from GRIB
%   TRANS(..)                   Matrices including transfer
%                               functions for roll and heave/
%                               pitch for different frequencies
%                               heading and velocity.
%   NOHEAD                      Number of heading in transfer
%                               file
%   NOFREQ                      Number of frequencies in
%                               transfer files
%   Lpp                         Length per perpendiculars
%   GCFC                        Fuel consumption for GCR
%   GClats                      Latitude vector for GCR
%   GClons                      Longitude vector for GCR
%---------------------Output--------------------------------
%   FC                          Fuel consumption for opt. route
%---------------------Internal--------------------------------
%   i                           Counting integer 
%   j                           Counting integer 
%   k                           Counting integer 
%   latC                        Vector for latitude values of 
%                               center points
%   lonC                        Vector for longitude values of 
%                               center points
%   lats                        Matrix with all latitude values in
%                               the feasible region
%   lons                        Matrix with all longitude values in
%                               the feasible region
%   Hs                          Significant wave height for 
%                               the given arc
%   T1                          Mean wave period for the  
%                               given arc
%   Tp                          Peak wave period for the  
%                               given arc
%   head                        Relative heading between wave
%                               and vessel or given arc
%   V                           Ship's speed
%   arcTime(S/R/L)              Matrix with traveled time for
%                               all arcs. One for straight, left and 
%                               right 
%   arcFuel(S/R/L)              Matrix with fuel consumption for
%                               all arcs One for straight, left and 
%                               right 
%   from(S/R/L)                 Departure nodes in sparse matrix
%   to(S/R/L)                   Arrival nodes in sparse matrix
%   fuel(S/R/L)                 Fuel cost between from and to
 
 
function [FC]=optfuel2(startLat,startLon,endLat,endLon,NumCenPts,Pb,...
 DeltaNM,bsfc,AVspeed,lon,lat,time,Uwind,Vwind,...
 SWH,MWD,MWP,ProbSlamming,ProbGreenWater,VAccFP,...
 RMSRoll,TRANSR2,TRANSR4,TRANSR6,TRANSR8,TRANSR10,TRANSHP,NOFREQ,...
 NOHEAD,Lpp,Xp,freeboard,AddedRes,C_wind,R_calm,...
 OpenWater,A_T,Eta_M,C_wake,C_t,Eta_R,D,GClats,GClons);
 
 
%-------------------------------------------------------------------
%Create the network of nodes. Ref step 1 in fig. 3.4 in thesis
%------------------------------------------------------------------
[GClat,GClon]=gcwaypts(startLat,startLon,endLat,endLon,50);
%Finding the center point in the route, step 1 in algorithm 3.2 in
%the thesis
CenterLat=GClat(26);
CenterLon=GClon(26);
%Finding the course between center point, and one before
%center point, step 2 in algorithm 3.2 in thesis
Course=azimuth('rh',GClat(25),GClon(25),CenterLat,CenterLon);
 
%Calculating point along perpendicular to center point, step 4
%in algorithm 3.2 in thesis
%Points above center line
for i=1:(NumCenPts-1)/2
    [latCenterN(i,1),lonCenterN(i,1)]=reckon('rh',CenterLat,...
    CenterLon,i*nm2deg(DeltaNM),(Course-90));
end
%Points below centerline
for i=1:(NumCenPts-1)/2
    [latCenterS(i,1),lonCenterS(i,1)]=reckon('rh',CenterLat,...
    CenterLon,i*nm2deg(DeltaNM),(Course+90));
end
 
%Creating one vector with center points for both above
%and under the line
for i=1:NumCenPts
    if i<(NumCenPts+1)/2
        latC(i,1)=latCenterN((NumCenPts+1)/2-i,1);
        lonC(i,1)=lonCenterN((NumCenPts+1)/2-i,1);
    elseif i==(NumCenPts+1)/2
        latC(i,1)=CenterLat;
        lonC(i,1)=CenterLon;
    else
        latC(i,1)=latCenterS(i-(NumCenPts+1)/2,1);
        lonC(i,1)=lonCenterS(i-(NumCenPts+1)/2,1);
    end
end
 
latC;
lonC;
 
%Defining point along the rhumb lines between start and center
%points and center points and end. Step 6 in algorithm 3.2
%in thesis.
for i=1:NumCenPts
    %points for first half
    [arclen1(i,1),az1(i,1)] = distance('rh',startLat,startLon,...
    latC(i),lonC(i));
    [lat1,lon1] = track2('rh',startLat,startLon,latC(i),lonC(i),...
    referenceSphere('unit sphere'),'degrees',round(arclen1(1,1)));
    lats1(:,i)=lat1;
    lons1(:,i)=lon1;
    
    %points for second half
    [arclen2(i,1),az2(i,1)] = distance('rh',latC(i),lonC(i),...
    endLat,endLon);
    [lat2,lon2] = track2('rh',latC(i),lonC(i),endLat,endLon,...
    referenceSphere('unit sphere'),'degrees',round(arclen1(1,1)));
    lats2(:,i)=lat2;
    lons2(:,i)=lon2;
end
 
%Creating two matrices with all nodes in the feasible region.
%Where the column represents the number of center points
%and the rows are the stages in the voyage, i.e. the same time
%step. Ref. fig 3.8 in the thesis.
s=length(lats1(:,1));
 
for i=1:(2*s-1)
    if i<s
        lats(i,:)=lats1(i,:);
        lons(i,:)=lons1(i,:);
    else
        lats(i,:)=lats2(i-s+1,:);
        lons(i,:)=lons2(i-s+1,:);
    end
end
lats;
lons;
 
%----------------------------------------------------------------
%Defining arcs between nodes. Three matrices created, one 
%for choosing the node to the left, one for straight, and 
%for choosing the node to the right. The definitions 
%of right and left is based on how the lats and lons vectors 
%are set up. The actual direction is dependent on the sailing 
%route.
 
%Second part is to add a cost to the given arc. This cost is 
%the fuel consumption which is calculated by the weather and 
%available speed. If the arc does not satisfy the seakeeping 
%criteria the fuel consumption is set to 1000 so that this arc 
%will not be chosen.
 
%This part of optfuel refers to the second and third step
%in fig. 3.4 in the thesis
 
for i=1:length(lats)-1
    for j=1:NumCenPts
        %------------------------------------------------------
        %Defining arcs and cost for going straight
        %------------------------------------------------------
        [arcStraight(i,j),azS(i,j)]=distance('gc',lats(i,j),...
        lons(i,j),lats(i+1,j),lons(i+1,j));
        %Finding the index to the latitude and longitude 
        %weather vector
        [~, idxlat] = min(abs(lat - lats(i+1,j)));
        [~, idxlon] = min(abs(lon - lons(i+1,j)));
        az = azimuth(lats(i,j),lons(i,j),lats(i+1,j),lons(i+1,j));
        %Finding Hs, T1 and direction for the given arc
        %Hs, T1 and dir is assigned value if NaN 
        idxtime=round(i/2);
        Hs=SWH(idxtime,idxlat,idxlon);
               if Hs<1 || isnan(Hs)    
            Hs=1;
        end
        T1=MWP(idxtime,idxlat,idxlon);
        if  isnan(T1)
            T1=5.4;
        end
        Tp=T1*1.3;
        dir=MWD(idxtime,idxlat,idxlon);
        if  isnan(dir)
            dir=90;
        end
        U_wind=Uwind(idxtime,idxlat,idxlon);
        V_wind=Vwind(idxtime,idxlat,idxlon);
        %Finding relative heading between wave and vessel
        head=sqrt(az^2+dir^2-2*az*dir);
            if head>180
                head=360-head;
            end   
        headS(i,j)=head;
        TpS(i,j)=Tp;
        HsS(i,j)=Hs;
        
        %Assigning the speed to the arc for the given weather 
        %condition
        [V,head_wind,Vel_wind]=calculateSpeed(Hs,T1,head,U_wind,...
            V_wind,AddedRes,C_wind,R_calm,...
            OpenWater,C_wake,C_t,A_T,Eta_M,Eta_R,D,Pb,az);
        
        VS(i,j)=V;
        Head_windS(i,j)=head_wind;
        Vel_windS(i,j)=Vel_wind;        
        
        %Assigning the fuel cost for the given arc
        arcTimeS(i,j)=deg2nm(arcStraight(i,j))/V;
        arcFuelS(i,j)=Pb*arcTimeS(i,j)*bsfc/1000000; 
        
        %Seakeeping function, checking criteria
        [RouteOK,ProbSlam,ProbGW,RMSRelVertAcc,RMSroll]=...
        seakeeping(T1,Hs,V,head,ProbSlamming,ProbGreenWater,...
        VAccFP,RMSRoll,TRANSR2,TRANSR4,TRANSR6,TRANSR8,...
        TRANSR10,TRANSHP,NOFREQ,NOHEAD,Lpp,Xp,freeboard);
        %Giving the arcFuel a value of 1000 if the routeOK 
        %is not satisfied
        if RouteOK==0
            arcFuelS(i,j)=1000;
        end
        ProbSlamS(i,j)=ProbSlam;
        ProbGWS(i,j)=ProbGW;
        RMSRelVertAccS(i,j)=RMSRelVertAcc;
        RMSrollS(i,j)=RMSroll;
        
        
        %-------------------------------------------------------
        %Defining arcs and cost for going right
        %------------------------------------------------------
        if j<NumCenPts   
        [arcRight(i,j),azR(i,j)]=distance('gc',lats(i,j),lons(i,j),...
        lats(i+1,j+1),lons(i+1,j+1));
        %Finding the index to the latitude and longitude weather vector
        [~, idxlat] = min(abs(lat - lats(i+1,j+1)));
        [~, idxlon] = min(abs(lon - lons(i+1,j+1)));
        az = azimuth(lats(i,j),lons(i,j),lats(i+1,j+1),lons(i+1,j+1));
        %Finding Hs, T1 and direction for the given arc
        idxtime=round(i/2);
        Hs=SWH(idxtime,idxlat,idxlon);
                if Hs<1 || isnan(Hs)    
            Hs=1;
        end
        T1=MWP(idxtime,idxlat,idxlon);
        if isnan(T1)
            T1=5.4;
        end
        Tp=T1*1.3;
        dir=MWD(idxtime,idxlat,idxlon);
        if isnan(dir)
            dir=90;
        end
        U_wind=Uwind(idxtime,idxlat,idxlon);
        V_wind=Vwind(idxtime,idxlat,idxlon);
        %Finding relative heading between wave and vessel
        head=sqrt(az^2+dir^2-2*az*dir);
            if head>180
                head=360-head;
            end  
        headR(i,j)=head;
        TpR(i,j)=Tp;
        HsR(i,j)=Hs;
        
                
        %Assigning the speed to the arc for the given weather 
        %condition
        [V,head_wind,Vel_wind]=calculateSpeed(Hs,T1,head,U_wind,...
            V_wind,AddedRes,C_wind,R_calm,...
            OpenWater,C_wake,C_t,A_T,Eta_M,Eta_R,D,Pb,az);
        
        %Assiging the fuel cost for the given arc
        arcTimeR(i,j)=deg2nm(arcRight(i,j))/V;
        arcFuelR(i,j)=Pb*arcTimeR(i,j)*bsfc/1000000; 
        
        %Seakeeping function, checking criteria
        [RouteOK,ProbSlam,ProbGW,RMSRelVertAcc,RMSroll]=...
        seakeeping(T1,Hs,V,head,ProbSlamming,ProbGreenWater,...
        VAccFP,RMSRoll,TRANSR2,TRANSR4,TRANSR6,TRANSR8,...
        TRANSR10,TRANSHP,NOFREQ,NOHEAD,Lpp,Xp,freeboard);
        %Giving the arcFuel a value of 1000 if the routeOK is 
        %not satisfied
        if RouteOK==0
            arcFuelR(i,j)=1000;
        end
        ProbSlamR(i,j)=ProbSlam;
        ProbGWR(i,j)=ProbGW;
        RMSRelVertAccR(i,j)=RMSRelVertAcc;
        RMSrollR(i,j)=RMSroll;
        end
        
        if j>1
        %-------------------------------------------------------
        %Defining arcs and costs for going right
        %------------------------------------------------------
        [arcLeft(i,j),azL(i,j)]=distance('gc',lats(i,j),lons(i,j),...
        lats(i+1,j-1),lons(i+1,j-1));
        %Finding the index to the latitude and longitude weather vector
        [~, idxlat] = min(abs(lat - lats(i+1,j-1)));
        [~, idxlon] = min(abs(lon - lons(i+1,j-1)));
        az = azimuth(lats(i,j),lons(i,j),lats(i+1,j-1),lons(i+1,j-1));
        %Finding Hs, T1 and direction for the given arc
        idxtime=round(i/2);
        Hs=SWH(idxtime,idxlat,idxlon);
                if Hs<1 || isnan(Hs)    
        Hs=1;
        end
        T1=MWP(idxtime,idxlat,idxlon);
        if isnan(T1)
            T1=5.4;
        end
        Tp=T1*1.3;
        dir=MWD(idxtime,idxlat,idxlon);
        if isnan(dir)
            dir=90;
        end
        U_wind=Uwind(idxtime,idxlat,idxlon);
        V_wind=Vwind(idxtime,idxlat,idxlon);
        %Finding relative heading between wave and vessel
        head=sqrt(az^2+dir^2-2*az*dir);
        if head>180
            head=360-head;
        end   
        headL(i,j)=head;
        TpL(i,j)=Tp;
        HsL(i,j)=Hs;
 
 
        %Assigning the speed to the arc for the given weather 
        %condition
        [V,head_wind,Vel_wind]=calculateSpeed(Hs,T1,head,U_wind,...
            V_wind,AddedRes,C_wind,R_calm,...
            OpenWater,C_wake,C_t,A_T,Eta_M,Eta_R,D,Pb,az);
 
        %Assiging the fuel cost for the given arc
        arcTimeL(i,j)=deg2nm(arcLeft(i,j))/V;
        arcFuelL(i,j)=Pb*arcTimeL(i,j)*bsfc/1000000;   %Fuel consumption in tonn
 
        %Seakeeping function, checking criteria
        [RouteOK,ProbSlam,ProbGW,RMSRelVertAcc,RMSroll]=...
        seakeeping(T1,Hs,V,head,ProbSlamming,ProbGreenWater,...
        VAccFP,RMSRoll,TRANSR2,TRANSR4,TRANSR6,TRANSR8,...
        TRANSR10,TRANSHP,NOFREQ,NOHEAD,Lpp,Xp,freeboard);
        %Giving the arcFuel a value of 1000 if the routeOK is not satisfied
        if RouteOK==0
            arcFuelL(i,j)=1000;
        end 
        ProbSlamL(i,j)=ProbSlam;
        ProbGWL(i,j)=ProbGW;
        RMSRelVertAccL(i,j)=RMSRelVertAcc;
        RMSrollL(i,j)=RMSroll;
        end
    end
end
 
arcFuelS;
arcFuelR;
arcFuelL(:,1)=[];
 
%Next section is used to verify the different arcs and their 
%calculated values
%Matrices corresponding to the straight arcs:
arcStraight; %Arc lengths
headS;        
HsS;
TpS;
VS;
 
Head_windS;
Vel_windS;
 
%Optinal to write values to excel spreadsheet
% xlswrite('optfuel2.xls',Head_windS,'head')
% xlswrite('optfuel2.xls',Vel_windS,'Vel')
% xlswrite('optfuel2.xls',VS,'V')
 
%Responses
ProbSlamS;
ProbGWS;
RMSRelVertAccS;
RMSrollS;
 
%Matrices corresponding to the right arcs:
arcRight;
headR;
HsR;
TpR;
%responses
ProbSlamR;
ProbGWR;
RMSRelVertAccR;
RMSrollR;
 
%Matrices corresponding to left arcs
arcLeft;
headL;
HsL;
TpL;
%Responses
ProbSlamL;
ProbGWL;
RMSRelVertAccL;
RMSrollL;
 
%--------------------------------------------------------------
%Optimization. The last part of the optfuel function refers
%to the fourth step in fig. 3.4. To use Dijstras algorithm 
%in Matlab a sparse matrix must be defined. This sparse matrix
%includes a vector with nodes that are departed (from) and nodes 
%that are arrival nodes (to). The sparse matrix also includes a
%vector of the cost of going from to. This part of the function is 
%difficult to understand, mostly trixing with matrices and vectors
%See fig 3.8 in thesis to understand the setup. 
%--------------------------------------------------------------
%Assigns an actual number to each node
for i=1:length(lats)-1
    for j=1:NumCenPts
        nodeS(i,j)=j+1+(i-1)*NumCenPts;
    end
end
nodeS;
nodeR=nodeS;
nodeR(:,1)=[];
nodeL=nodeS;
nodeL(:,NumCenPts)=[];
 
SizeNodes=size(nodeS);
NumberNodes=SizeNodes(1,1)*SizeNodes(1,2);
 
%Node connection
%Define the straight points
for i=1:((length(lats(:,1))-1)*NumCenPts)
    if i<NumCenPts+1;
        fromS(i)=1;
    else
    fromS(i)=i-NumCenPts+1;
    end
end
 
 
%Defining to nodes for straight arcs and cost
for i=1:((length(lats(:,1))-1)*NumCenPts)
    if i<NumberNodes-NumCenPts+1
    toS(i)=1+i;
    fuelS(i)=arcFuelS(nodeS==(i+1));
    else
        toS(i)=((length(lats(:,1)))-2)*NumCenPts+2;
        fuelS(i)=arcFuelS(nodeS==(i+1));
    end
end
 
%Define the right arcs
for j=1:((length(lats(:,1))-2)*(NumCenPts-1))
    fromR(j)=j+1;
end
 
for j=1:((length(lats(:,1))-2)*(NumCenPts-1))
    if j<((length(lats(:,1))-2)*(NumCenPts))-NumCenPts+1
        toR(j)=NumCenPts+j+1;
    else
        toR(j)=((length(lats(:,1)))-2)*NumCenPts+2;
    end
    
end
 
%Editing the to and from nodes and cost for 
%right and left arcs
fromR;
toR;
fromL=fromR;
toL=toR;
fromR(NumCenPts:NumCenPts:end)=[];
toR(1:NumCenPts:end)=[];
fromL(1:NumCenPts:end)=[];
toL(NumCenPts:NumCenPts:end)=[];
 
%Assigning fuel cost for right and left arcs
for i=1:length(toR)
    fuelR(i)=arcFuelR(nodeR==toR(i));
    fuelL(i)=arcFuelL(nodeL==toL(i));
end
 
%Defining the final to, from and cost vectors to be
%used in the sparse matrix.
for i =1:length(fuelS)+length(fuelR)+length(fuelL)
    if i<length(fuelS)+1
        fuel(i)=fuelS(i);
        from(i)=fromS(i);
        to(i)=toS(i);
    elseif i<length(fuelS)+length(fuelR)+1
        fuel(i)=fuelR(i-length(fuelS));
        from(i)=fromR(i-length(fuelS));
        to(i)=toR(i-length(fuelS));
    else
        fuel(i)=fuelL(i-length(fuelS)-length(fuelR));
        from(i)=fromL(i-length(fuelS)-length(fuelR));
        to(i)=toL(i-length(fuelS)-length(fuelR));
    end
end
%Defining a sparse to use in shortest path function
SparseGraph = sparse(from,to,fuel);
MatrixGraph = full(SparseGraph);
%Adding an additional row to the full matrix so that the matrix 
%is symetrical. The added row is defined as an edge.
MatrixGraph((length(lats)-2)*NumCenPts+2,:)=0;
%Converting matrix back to sparse
SparseGraph=sparse(MatrixGraph);
 
%Calculating the fuel consumption and path for the 
%optimization problem with Dijkstra's algorithm
[FC,path,pred] = graphshortestpath(SparseGraph,1,...
(length(lats)-2)*NumCenPts+2,'Method','Dijkstra');
 
FC;
path;
 
%Plotting the graph of nodes and connecting arcs
h = view(biograph(SparseGraph,[],'ShowWeights','on'))
set(h.Nodes(path),'Color',[1 0.4 0.4])
edges = getedgesbynodeid(h,get(h.Nodes(path),'ID'));
set(edges,'LineColor',[1 0 0])
set(edges,'LineWidth',1.5)
 
%Assigning geographical coordinates to the nodes in 
%order to plot in map
 
for i=2:length(lats)-1
   %locating index for nodes via path
   [~,indnodeS]=find(nodeS(i-1,:)==path(i));
   OptFuelPathLat(i,1)=lats(i,indnodeS);
   OptFuelPathLon(i,1)=lons(i,indnodeS);
end
OptFuelPathLat(1,1)=startLat;
OptFuelPathLon(1,1)=startLon;
OptFuelPathLat(length(lats),1)=endLat;
OptFuelPathLon(length(lats),1)=endLon;
 
 
%Map plot with nodes and route
S=8;
figure (2)
geoshow('landareas.shp', 'FaceColor', [0.5 1.0 0.5]);
hold on
geoshow(GClats,GClons)
hold on
geoshow(OptFuelPathLat,OptFuelPathLon,'displaytype','line','color','r')
 
for i=1:NumCenPts
hold on
scatter(lons(:,i),lats(:,i),S,'fill','b')
end
 
hold on
scatter(lonC,latC,S,'fill','r')
 
 
end

