%------------------------------------------------------------
%               readinput.m
%               Hege Eskild
%               Spring 2014
%
% This function reads the input transfer files from Shipx.
% Output is matrices including transfer function for different
% wave headings, frequencies and velocities. One transfer 
% matrix is calculated for heave and pitch, and several for
% roll. The functions includes both the real and imaginary
% part of the transfer function. For the program to operate
% properly all transfer input files must be of the same size.
% i.e. calculated for the same velocities, headings and
% frequencies. Several for-loops are used to assign the right
% value to the right frequency, heading and velocity.
%
%------------------------------------------------------------
%   Variable name               Definition
%----------------------Input-----------------------------
%   TRANSinput(..)              Transfer files from ShipX (.re1)
%---------------------Output--------------------------------
%   TRANS(..)                   Matrices including transfer
%                               functions for roll and heave/
%                               pitch for different frequencies
%                               heading and velocity.
%   NOHEAD                      Number of heading in transfer
%                               file
%   NOFREQ                      Number of frequencies in
%                               transfer files
%   Lpp                         Length per perpendiculars
%---------------------Internal--------------------------------
%   j                           Counting integer 
%   i                           Counting integer
%   k                           Counting integer
%   NOVEL                       Number of velocities
%   NOHEAD                      Number of headings 
%   NOFREQ                      Number of frequencies
%   NDOF                        Number of degrees of freedom
 
 
function [TRANSR2,TRANSR4,TRANSR6,TRANSR8,TRANSR10,TRANSHP,...
         NOFREQ,NOHEAD,Lpp]=readinput(TRANSinputHP,...
         TRANSinputH2,TRANSinputH4,TRANSinputH6,...
         TRANSinputH8,TRANSinputH10);
 
%--------------------------------------------------------     
%Reading ShipX file for heave and pitch transfer function
%--------------------------------------------------------
 
info=fscanf(TRANSinputHP,'%c',[1,500]);
 
%Defining vessel constants 
rho=fscanf(TRANSinputHP,'%f',1);
g=fscanf(TRANSinputHP,'%f',1);
Lpp=fscanf(TRANSinputHP,'%f',1);
B=fscanf(TRANSinputHP,'%f',1);
T=fscanf(TRANSinputHP,'%f',1);
LCG=fscanf(TRANSinputHP,'%f',1);
VCG=fscanf(TRANSinputHP,'%f',1);
 
%Defining number of velocities, headings, frequencies
%and degrees of freedom.
NOVEL=fscanf(TRANSinputHP,'%f',1);
NOHEAD=fscanf(TRANSinputHP,'%f',1); 
NOFREQ=fscanf(TRANSinputHP,'%f',1);
NDOF=fscanf(TRANSinputHP,'%f',1);
 
%Defining size of matrix
TRANSRHP=zeros(NOFREQ*NOHEAD*NOVEL,7);
 
for k=1:NOVEL
VEL=fscanf(TRANSinputHP,'%f',1);
SINK=fscanf(TRANSinputHP,'%f',1);
TRIM=fscanf(TRANSinputHP,'%f',1);
XMTN=fscanf(TRANSinputHP,'%f',1); 
ZMTN=fscanf(TRANSinputHP,'%f',1);
  
for j=1:NOHEAD
HEAD=fscanf(TRANSinputHP,'%f',1);
 
for i=1:NOFREQ
FREQ=fscanf(TRANSinputHP,'%f',1);
trans=fscanf(TRANSinputHP,'%f %f %f',[3,NDOF]);
 
%Assigning velocity, heading, frequency, and real and 
%imaginary part of heave and pith to correct row and
%column in heave/pitch transfer matrix.
TRANSHP((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,1)=VEL/(1852/3600);
TRANSHP((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,2)=HEAD;
TRANSHP((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,3)=FREQ;
TRANSHP((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,4)=trans(2,3);
TRANSHP((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,5)=trans(3,3);
TRANSHP((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,6)=trans(2,5);
TRANSHP((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,7)=trans(3,5);
 
end
    
end
 
end
TRANSHP;
     
%--------------------------------------------------------     
%Reading ShipX file for roll transfer function H=2
%--------------------------------------------------------
info=fscanf(TRANSinputH2,'%c',[1,500]);
 
%Defining vessel constants 
rho=fscanf(TRANSinputH2,'%f',1);
g=fscanf(TRANSinputH2,'%f',1);
Lpp=fscanf(TRANSinputH2,'%f',1);
B=fscanf(TRANSinputH2,'%f',1);
T=fscanf(TRANSinputH2,'%f',1);
LCG=fscanf(TRANSinputH2,'%f',1);
VCG=fscanf(TRANSinputH2,'%f',1);
 
%Defining number of velocities, headings, frequencies
%and degrees of freedom.
NOVEL=fscanf(TRANSinputH2,'%f',1);
NOHEAD=fscanf(TRANSinputH2,'%f',1); 
NOFREQ=fscanf(TRANSinputH2,'%f',1);
NDOF=fscanf(TRANSinputH2,'%f',1);
 
%Defining size of matrix
TRANSR2=zeros(NOFREQ*NOHEAD*NOVEL,4);
 
for k=1:NOVEL
VEL=fscanf(TRANSinputH2,'%f',1);
SINK=fscanf(TRANSinputH2,'%f',1);
TRIM=fscanf(TRANSinputH2,'%f',1);
XMTN=fscanf(TRANSinputH2,'%f',1); 
ZMTN=fscanf(TRANSinputH2,'%f',1);
  
for j=1:NOHEAD
HEAD=fscanf(TRANSinputH2,'%f',1);
 
for i=1:NOFREQ
FREQ=fscanf(TRANSinputH2,'%f',1);
trans=fscanf(TRANSinputH2,'%f %f %f',[3,NDOF]);
 
%Assigning velocity, heading, frequency, and real and 
%part of roll to correct row and
%column in roll H=2 transfer matrix.
TRANSR2((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,1)=VEL/(1852/3600);
TRANSR2((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,2)=HEAD;
TRANSR2((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,3)=FREQ;
TRANSR2((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,4)=trans(2,4);
 
end
    
end
 
end
TRANSR2;
     
%--------------------------------------------------------     
%Reading ShipX file for roll transfer function H=4
%--------------------------------------------------------    
info=fscanf(TRANSinputH4,'%c',[1,500]);
 
%Defining vessel constants 
rho=fscanf(TRANSinputH4,'%f',1);
g=fscanf(TRANSinputH4,'%f',1);
Lpp=fscanf(TRANSinputH4,'%f',1);
B=fscanf(TRANSinputH4,'%f',1);
T=fscanf(TRANSinputH4,'%f',1);
LCG=fscanf(TRANSinputH4,'%f',1);
VCG=fscanf(TRANSinputH4,'%f',1);
 
%Defining number of velocities, headings, frequencies
%and degrees of freedom.
NOVEL=fscanf(TRANSinputH4,'%f',1);
NOHEAD=fscanf(TRANSinputH4,'%f',1); 
NOFREQ=fscanf(TRANSinputH4,'%f',1);
NDOF=fscanf(TRANSinputH4,'%f',1);
 
%Defining size of matrix
TRANSR4=zeros(NOFREQ*NOHEAD*NOVEL,4);
 
for k=1:NOVEL
VEL=fscanf(TRANSinputH4,'%f',1);
SINK=fscanf(TRANSinputH4,'%f',1);
TRIM=fscanf(TRANSinputH4,'%f',1);
XMTN=fscanf(TRANSinputH4,'%f',1); 
ZMTN=fscanf(TRANSinputH4,'%f',1);
  
for j=1:NOHEAD
HEAD=fscanf(TRANSinputH4,'%f',1);
 
for i=1:NOFREQ
FREQ=fscanf(TRANSinputH4,'%f',1);
trans=fscanf(TRANSinputH4,'%f %f %f',[3,NDOF]);
 
%Assigning velocity, heading, frequency, and real and 
%part of roll to correct row and
%column in roll H=4 transfer matrix.
TRANSR4((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,1)=VEL/(1852/3600);
TRANSR4((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,2)=HEAD;
TRANSR4((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,3)=FREQ;
TRANSR4((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,4)=trans(2,4);
 
end
    
end
 
end
TRANSR4;
 
%--------------------------------------------------------     
%Reading ShipX file for roll transfer function H=6
%--------------------------------------------------------
info=fscanf(TRANSinputH6,'%c',[1,500]);
 
%Defining vessel constants 
rho=fscanf(TRANSinputH6,'%f',1);
g=fscanf(TRANSinputH6,'%f',1);
Lpp=fscanf(TRANSinputH6,'%f',1);
B=fscanf(TRANSinputH6,'%f',1);
T=fscanf(TRANSinputH6,'%f',1);
LCG=fscanf(TRANSinputH6,'%f',1);
VCG=fscanf(TRANSinputH6,'%f',1);
 
%Defining number of velocities, headings, frequencies
%and degrees of freedom.
NOVEL=fscanf(TRANSinputH6,'%f',1);
NOHEAD=fscanf(TRANSinputH6,'%f',1); 
NOFREQ=fscanf(TRANSinputH6,'%f',1);
NDOF=fscanf(TRANSinputH6,'%f',1);
 
%Defining size of matrix
TRANSR6=zeros(NOFREQ*NOHEAD*NOVEL,4);
 
for k=1:NOVEL
VEL=fscanf(TRANSinputH6,'%f',1);
SINK=fscanf(TRANSinputH6,'%f',1);
TRIM=fscanf(TRANSinputH6,'%f',1);
XMTN=fscanf(TRANSinputH6,'%f',1); 
ZMTN=fscanf(TRANSinputH6,'%f',1);
  
for j=1:NOHEAD
HEAD=fscanf(TRANSinputH6,'%f',1);
 
for i=1:NOFREQ
FREQ=fscanf(TRANSinputH6,'%f',1);
trans=fscanf(TRANSinputH6,'%f %f %f',[3,NDOF]);
 
%Assigning velocity, heading, frequency, and real and 
%part of roll to correct row and
%column in roll H=6 transfer matrix.
TRANSR6((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,1)=VEL/(1852/3600);
TRANSR6((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,2)=HEAD;
TRANSR6((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,3)=FREQ;
TRANSR6((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,4)=trans(2,4);
 
end
    
end
 
end
TRANSR6;
 
%--------------------------------------------------------     
%Reading ShipX file for roll transfer function H=8
%--------------------------------------------------------
info=fscanf(TRANSinputH8,'%c',[1,500]);
 
%Defining vessel constants 
rho=fscanf(TRANSinputH8,'%f',1);
g=fscanf(TRANSinputH8,'%f',1);
Lpp=fscanf(TRANSinputH8,'%f',1);
B=fscanf(TRANSinputH8,'%f',1);
T=fscanf(TRANSinputH8,'%f',1);
LCG=fscanf(TRANSinputH8,'%f',1);
VCG=fscanf(TRANSinputH8,'%f',1);
 
%Defining number of velocities, headings, frequencies
%and degrees of freedom.
NOVEL=fscanf(TRANSinputH8,'%f',1);
NOHEAD=fscanf(TRANSinputH8,'%f',1); 
NOFREQ=fscanf(TRANSinputH8,'%f',1);
NDOF=fscanf(TRANSinputH8,'%f',1);
 
%Defining size of matrix
TRANSR8=zeros(NOFREQ*NOHEAD*NOVEL,4);
 
for k=1:NOVEL
VEL=fscanf(TRANSinputH8,'%f',1);
SINK=fscanf(TRANSinputH8,'%f',1);
TRIM=fscanf(TRANSinputH8,'%f',1);
XMTN=fscanf(TRANSinputH8,'%f',1); 
ZMTN=fscanf(TRANSinputH8,'%f',1);
  
for j=1:NOHEAD
HEAD=fscanf(TRANSinputH8,'%f',1);
 
for i=1:NOFREQ
FREQ=fscanf(TRANSinputH8,'%f',1);
trans=fscanf(TRANSinputH8,'%f %f %f',[3,NDOF]);
 
%Assigning velocity, heading, frequency, and real and 
%part of roll to correct row and
%column in roll H=8 transfer matrix.
TRANSR8((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,1)=VEL/(1852/3600);
TRANSR8((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,2)=HEAD;
TRANSR8((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,3)=FREQ;
TRANSR8((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,4)=trans(2,4);
 
end
    
end
 
end
TRANSR8;
 
%--------------------------------------------------------     
%Reading ShipX file for roll transfer function H=10
%--------------------------------------------------------
info=fscanf(TRANSinputH10,'%c',[1,500]);
 
%Defining vessel constants 
rho=fscanf(TRANSinputH10,'%f',1);
g=fscanf(TRANSinputH10,'%f',1);
Lpp=fscanf(TRANSinputH10,'%f',1);
B=fscanf(TRANSinputH10,'%f',1);
T=fscanf(TRANSinputH10,'%f',1);
LCG=fscanf(TRANSinputH10,'%f',1);
VCG=fscanf(TRANSinputH10,'%f',1);
 
%Defining number of velocities, headings, frequencies
%and degrees of freedom.
NOVEL=fscanf(TRANSinputH10,'%f',1);
NOHEAD=fscanf(TRANSinputH10,'%f',1); 
NOFREQ=fscanf(TRANSinputH10,'%f',1);
NDOF=fscanf(TRANSinputH10,'%f',1);
 
%Defining size of matrix
TRANSR10=zeros(NOFREQ*NOHEAD*NOVEL,4);
 
for k=1:NOVEL
VEL=fscanf(TRANSinputH10,'%f',1);
SINK=fscanf(TRANSinputH10,'%f',1);
TRIM=fscanf(TRANSinputH10,'%f',1);
XMTN=fscanf(TRANSinputH10,'%f',1); 
ZMTN=fscanf(TRANSinputH10,'%f',1);
 
for j=1:NOHEAD
HEAD=fscanf(TRANSinputH10,'%f',1);
 
for i=1:NOFREQ
FREQ=fscanf(TRANSinputH10,'%f',1);
trans=fscanf(TRANSinputH10,'%f %f %f',[3,NDOF]);
 
%Assigning velocity, heading, frequency, and real and 
%part of roll to correct row and
%column in roll H=10 transfer matrix.
TRANSR10((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,1)=VEL/(1852/3600);
TRANSR10((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,2)=HEAD;
TRANSR10((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,3)=FREQ;
TRANSR10((k-1)*NOHEAD*NOFREQ+(j-1)*NOFREQ+i,4)=trans(2,4);
 
end
    
end
 
end
TRANSR10;
 
 
end
 