%------------------------------------------------------------
%               seakeeping.m
%               Hege Eskild
%               Spring 2014
%
% The seakeeping function calculates the standard deviation
% in roll and vertical acceleration, as well as the probability
% for slamming and deck wetness. The main output of this 
% function is a routeOK variable defining whether or not the
% given arc satisfies the seakeeping criteria. Output from this
% function is also the Seakeeping values used to evaluate the
% entire route and the feasible region.
%
%------------------------------------------------------------
%   Variable name               Definition
%----------------------Input-----------------------------
%   T1                          Mean wave period
%   Hs                          Significant wave height
%   V                           Vessel speed
%   head                        relative heading between waves and 
%                               vessel
%   ProbSlamming                Seakeeping criterion for slamming
%   ProbGreenWater              Seakeeping criterion for deck 
%                               wetness
%   VAccFP                      Seakeeping criterion for vertical 
%                               acceleration
%   RMSRoll                     Seakeeping criterion for roll
 
%   TRANS(..)                   Matrices including transfer
%                               functions for roll and heave/
%                               pitch for different frequencies
%                               heading and velocity.
%   NOHEAD                      Number of heading in transfer
%                               file
%   NOFREQ                      Number of frequencies in
%                               transfer files
%   Lpp                         Length per perpendiculars
%   Xp                          X coordinate of FP (negative from 
%                               CG)
%   freeboard                   freeboard at FP
%                               deduction
%---------------------Output--------------------------------
%   RouteOK                     Defines whether or not the seakeeping
%                               criteria are satisfied
%   ProbSlam                    Probability of slamming for arc
%   probGW                      Probability of green water for arc
%   RMSRelVertAcc               Standard deviation for relative 
%                               vertical acceleration for arc
%   RMSroll                     Standard deviation for roll for
%                               arc
%---------------------Internal--------------------------------
%   Tp                          Peak period
%   omegap                      Peak frequency
%   omega                       Frequency from ShipX file
%   m0,1,2,4Roll                Vectors for calculating 
%                               spectral moments for roll
%   m0,1,2,4RelVert             Vectors for calculating 
%                               spectral moments for relative
%                               vertical motions
%   sigma                       Spectral width parameter
%   gamma                       Peakedness parameter
 
 
function [RouteOK,ProbSlam,ProbGW,RMSRelVertAcc,RMSroll]=seakeeping...
    (T1,Hs,V,head,ProbSlamming,ProbGreenWater,VAccFP,...
    RMSRoll,TRANSR2,TRANSR4,TRANSR6,TRANSR8,TRANSR10,TRANSHP,NOFREQ,...
    NOHEAD,Lpp,Xp,freeboard)
 
%-------------------------------------------------------------------

%defining wariables for wave spectrum
ZETAa=Hs/2;
gamma=1;
Tp=1.3*T1;
omegap=2*pi()/Tp;
V=V+0.0001;
 
%------------------------------------------------------------
 
%First part calculates RMS for roll
 
%Choosing the correct transfer file from ShipX
if Hs<3
    TRANSROLL=TRANSR2;
elseif Hs<5
    TRANSROLL=TRANSR4;
elseif Hs<7
    TRANSROLL=TRANSR6;
elseif Hs<9
    TRANSROLL=TRANSR8;
else
    TRANSROLL=TRANSR10;
end
 
ROLL=zeros(NOFREQ,1);
m0ROLL=zeros(NOFREQ,1);
m1ROLL=zeros(NOFREQ,1);
m2ROLL=zeros(NOFREQ,1);
m4ROLL=zeros(NOFREQ,1);
 
%Defining step in velocity and heading from transfer
deltaV=(TRANSROLL(NOFREQ*NOHEAD+1,1)-TRANSROLL(1,1))/2;
deltaHEAD=(TRANSROLL(NOFREQ+1,2)-TRANSROLL(1,2))/2;
 
%finding minimum and maximum speed in trans file
minV=TRANSROLL(1,1);
maxV=TRANSROLL(length(TRANSROLL),1);
if V>maxV
    V=maxV-0.001;
elseif V<minV
    V=minV+0.001;
end
 
 
ind=zeros(NOFREQ,1);
%Locating correct transfer function index
for i=1:length(TRANSROLL)
   if (TRANSROLL(i,1)>(V-deltaV) && TRANSROLL(i,1)<(V+deltaV) &&...
      TRANSROLL(i,2)>(head-deltaHEAD) && TRANSROLL(i,2)<(head+deltaHEAD))
      ind(i)=i;
   end
end
ind(ind==0)=[];
ind;
 
%Calculating the motion spectrum with given heading, Hs and Tp.
%Using the JONSWAP spectrum formulation
for j=1:NOFREQ
    i=j+ind(1)-1;
    RAO=TRANSROLL(i,4);
    omegaR=TRANSROLL(i,3);
    omegae=omegaR-omegaR^2*V/9.81*cos(head/180*pi());
    if omegaR<omegap
        sigma=0.07;
    else
        sigma=0.09;
    end
    Y=-0.5*((omegaR-omegap)/(sigma*omegap))^2;
    sROLL=RAO^2*5.061*Hs^2/(Tp^4)*(1-0.287*log(gamma))...
        *9.81^2*omegaR^(-5)*...
        exp(-1.25*(omegap/omegaR)^4)*gamma^exp(Y);
 
 
m0ROLL(j)=sROLL;
m1ROLL(j)=omegaR*sROLL;
m2ROLL(j)=omegaR^2*sROLL;
m4ROLL(j)=omegaR^4*sROLL;
 
OmegaR(j,1)=omegaR;    
  
end
OmegaR;
m0ROLL;
h=(TRANSROLL(NOFREQ,3)-TRANSROLL(1,3))/NOFREQ;
%Calculating spectral moment M0 for roll
M0roll=trapz(OmegaR,m0ROLL);
 
RMSroll=sqrt(M0roll)*180/pi();
 
 
 
%-------------------------------------------------------------------------
 
%Third part calculates RMS Vertical acceleration at bow
 
%Define X and Y Position for calculating acceleration, 
%probability of slamming and probability of green water on deck.
Yp=0;
 
RelVert=zeros(NOFREQ,1);
m0RelVert=zeros(NOFREQ,1);
m1RelVert=zeros(NOFREQ,1);
m2RelVert=zeros(NOFREQ,1);
m4RelVert=zeros(NOFREQ,1);
 
%Defining step in velocity and heading from transfer
deltaV=(TRANSHP(NOFREQ*NOHEAD+1,1)-TRANSHP(1,1))/2;
deltaHEAD=(TRANSHP(NOFREQ+1,2)-TRANSHP(1,2))/2;
 
ind=zeros(NOFREQ,1);
 
 
%Locating correct transfer function index
for i=1:length(TRANSHP)
   if (TRANSHP(i,1)>(V-deltaV) && TRANSHP(i,1)<(V+deltaV) &&...
      TRANSHP(i,2)>(head-deltaHEAD) && TRANSHP(i,2)<(head+deltaHEAD))
      ind(i)=i;
   end
end
ind(ind==0)=[];
ind;
 
%Calculating the relative at FP motion spectrum with given 
%heading, Hs and Tp. Using the JONSWAP spectrum formulation
for j=1:NOFREQ
    i=j+ind(1)-1;
    omega=TRANSHP(i,3);
    %Defining real and imaginary part of heave and pitch
    heaveR=TRANSHP(i,4);
    heaveIm=TRANSHP(i,5);
    pitchR=TRANSHP(i,6);
    pitchIm=TRANSHP(i,7);
    
    %Defining phase angle for heave and pitch
    phaseHeave=atan(heaveIm/heaveR);
    phasePitch=atan(pitchIm/pitchR);
    
    %Calculating the three parts of the RAO
    RAO1=sqrt(heaveR^2+heaveIm^2)*exp(1i*phaseHeave);
    RAO2=Xp*sqrt(pitchR^2+pitchIm^2)*exp(1i*phasePitch);
    RAO3=exp(-1i*omega^2/9.81*(Xp*cos(head/180*pi())+...
         Yp*sin(head/180*pi())));
    
    RAO4=RAO1-RAO2-RAO3;
    %RAO used for calculations
    RAO=abs(RAO4);
    
        if j<NOFREQ
    Deltaomega=TRANSHP(j+1,3)-TRANSHP(j,3);
    else
    Deltaomega=TRANSHP(j,3)-TRANSHP(j-1,3);
    end
    
    if omega<omegap
        sigma=0.07;
    else
        sigma=0.09;
    end
    Y=-0.5*((omega-omegap)/(sigma*omegap))^2;
    sRelVert=RAO^2*5.061*(Hs^2)/(Tp^4)*(1-0.287*log(gamma))*9.81^2*omega^(-5)*...
        exp(-1.25*(omegap/omega)^4)*gamma^exp(Y);
    
    S(j,1)=5.061*(Hs^2)/(Tp^4)*(1-0.287*log(gamma))*9.81^2*omega^(-5)*...
        exp(-1.25*(omegap/omega)^4)*gamma^exp(Y);
    
m0RelVert(j)=sRelVert;
m1RelVert(j)=omega*sRelVert;
m2RelVert(j)=omega^2*sRelVert;
m4RelVert(j)=omega^4*sRelVert;
 
Omega(j,1)=omega;
SR(j,1)=sRelVert;
 
end
 
Omega;
SR;
 
%Calculating the motion standard deviation
m0RelVert;
M0RelVert=trapz(Omega,m0RelVert);
RMSRelVertMo=sqrt(M0RelVert);
 
%Calculating the velocity standard deviation
m2RelVert;
M2RelVert=trapz(Omega,m2RelVert);
RMSRelVertVel=sqrt(M2RelVert);
 
%Calculating the acceleration standard deviation
m4RelVert;
%M4RelVert=sum(m4RelVert)
M4RelVert=trapz(Omega,m4RelVert);
RMSRelVertAcc=sqrt(M4RelVert);
 
%------------------------------------------------------------------
%Probability of slamming
 
Vcr=0.093*sqrt(9.81*Lpp);
ProbSlam=exp(-(Vcr^2/(2*RMSRelVertVel^2)+11^2/(2*RMSRelVertMo^2)));
 
%------------------------------------------------------------------
%Probability of green water on deck
 
ProbGW=exp(-(freeboard)^2/(2*RMSRelVertMo^2));
 
 
%Defines whether the arc is an option
if ProbSlamming>ProbSlam &&...
    ProbGreenWater>ProbGW &&...
    VAccFP>RMSRelVertAcc &&...
    RMSRoll>RMSroll
    RouteOK=1 ;             %RouteOK equals 1 means optional
else
    RouteOK=0 ;              %RouteOK equals 0 delete option
    
end
 
end